/*
 * Copyright 2011, Maarten Baert <maarten-baert@hotmail.com>
 * 
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#ifndef GM_H
#define GM_H

#include <cstddef>
#include <cstring>
#include <climits>
#include <float.h>

#define gmexport extern "C" __declspec (dllexport)

inline bool gm_double_to_bool(double d) {
	return (d>=0.5);
}
inline int gm_double_to_int(double d) {
	if(d<=(double)(int)(INT_MIN)) return (int)(INT_MIN);
	if(d>=(double)(int)(INT_MAX)) return (int)(INT_MAX);
	return (int)(d);
}
inline unsigned int gm_double_to_uint(double d) {
	if(d<=(double)(unsigned int)(0)) return (unsigned int)(0);
	if(d>=(double)(unsigned int)(UINT_MAX)) return (unsigned int)(UINT_MAX);
	return (unsigned int)(d);
}
inline long long gm_double_to_longlong(double d) {
	if(d<=(double)(long long)(LLONG_MIN)) return (long long)(LLONG_MIN);
	if(d>=(double)(long long)(LLONG_MAX)) return (long long)(LLONG_MAX);
	return (long long)(d);
}
inline unsigned long long gm_double_to_ulonglong(double d) {
	if(d<=(double)(unsigned long long)(0)) return (unsigned long long)(0);
	if(d>=(double)(unsigned long long)(ULLONG_MAX)) return (unsigned long long)(ULLONG_MAX);
	return (unsigned long long)(d);
}
inline float gm_double_to_float(double d) {
	if(d<=(double)(float)(-FLT_MAX)) return (float)(-FLT_MAX);
	if(d>=(double)(float)(+FLT_MAX)) return (float)(+FLT_MAX);
	return (float)(d);
}

#endif // GM_H

