/*
 * Copyright 2011, Maarten Baert <maarten-baert@hotmail.com>
 * 
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include "gm.h"
#include "Buffer.h"

Buffer returnstring;

gmexport double shared_buffers_get_handle() {
	return (uintptr_t)(&shared_buffer_iterface);
}

gmexport double buffer_create() {
	Buffer *b = Buffer::Create();
	return Buffer::GetID(b);
}
gmexport double buffer_destroy(double id) {
	Buffer *b = Buffer::Find(gm_double_to_uint(id));
	if(b == NULL) {
		return 0;
	}
	Buffer::Destroy(b);
	return 1;
}
gmexport double buffer_exists(double id) {
	Buffer *b = Buffer::Find(gm_double_to_uint(id));
	return (b == NULL)? 0 : 1;
}

gmexport const char* buffer_to_string(double id) {
	Buffer *b = Buffer::Find(gm_double_to_uint(id));
	if(b == NULL) return 0;
	return Buffer::ToString(b);
}
gmexport double buffer_get_pos(double id) {
	Buffer *b = Buffer::Find(gm_double_to_uint(id));
	if(b == NULL) return 0;
	return Buffer::GetPos(b);
}
gmexport double buffer_get_length(double id) {
	Buffer *b = Buffer::Find(gm_double_to_uint(id));
	if(b == NULL) return 0;
	return Buffer::GetLength(b);
}
gmexport double buffer_at_end(double id) {
	Buffer *b = Buffer::Find(gm_double_to_uint(id));
	if(b == NULL) return 0;
	return (Buffer::IsAtEnd(b))? 1 : 0;
}
gmexport double buffer_get_error(double id) {
	Buffer *b = Buffer::Find(gm_double_to_uint(id));
	if(b == NULL) return 0;
	return (Buffer::GetError(b))? 1 : 0;
}
gmexport double buffer_clear_error(double id) {
	Buffer *b = Buffer::Find(gm_double_to_uint(id));
	if(b == NULL) return 0;
	Buffer::ClearError(b);
	return 1;
}

gmexport double buffer_clear(double id) {
	Buffer *b = Buffer::Find(gm_double_to_uint(id));
	if(b == NULL) return 0;
	Buffer::Clear(b);
	return 1;
}
gmexport double buffer_set_pos(double id, double newpos) {
	Buffer *b = Buffer::Find(gm_double_to_uint(id));
	if(b == NULL) return 0;
	Buffer::SetPos(b, gm_double_to_uint(newpos));
	return 1;
}
gmexport double buffer_set_length(double id, double newlength) {
	Buffer *b = Buffer::Find(gm_double_to_uint(id));
	if(b == NULL) return 0;
	unsigned int newlen = gm_double_to_uint(newlength);
	unsigned int oldlen = Buffer::GetLength(b);
	Buffer::SetLength(b, newlen);
	if(newlen > oldlen) {
		// GM users probably don't expect undefined behaviour,
		// so the new part of the buffer is filled with null bytes
		memset(Buffer::GetData(b) + oldlen, 0, newlen - oldlen);
	}
	return 1;
}

gmexport double buffer_read_from_file(double id, const char* filename) {
	Buffer *b = Buffer::Find(gm_double_to_uint(id));
	if(b == NULL) return 0;
	return (Buffer::ReadFromFile(b, filename))? 1 : 0;
}
gmexport double buffer_write_to_file(double id, const char* filename) {
	Buffer *b = Buffer::Find(gm_double_to_uint(id));
	if(b == NULL) return 0;
	return (Buffer::WriteToFile(b, filename))? 1 : 0;
}

gmexport double buffer_read_int8(double id) {
	Buffer *b = Buffer::Find(gm_double_to_uint(id));
	if(b == NULL) return 0;
	return (double)(Buffer::ReadX<int8_t>(b));
}
gmexport double buffer_read_uint8(double id) {
	Buffer *b = Buffer::Find(gm_double_to_uint(id));
	if(b == NULL) return 0;
	return (double)(Buffer::ReadX<uint8_t>(b));
}
gmexport double buffer_read_int16(double id) {
	Buffer *b = Buffer::Find(gm_double_to_uint(id));
	if(b == NULL) return 0;
	return (double)(Buffer::ReadX<int16_t>(b));
}
gmexport double buffer_read_uint16(double id) {
	Buffer *b = Buffer::Find(gm_double_to_uint(id));
	if(b == NULL) return 0;
	return (double)(Buffer::ReadX<uint16_t>(b));
}
gmexport double buffer_read_int32(double id) {
	Buffer *b = Buffer::Find(gm_double_to_uint(id));
	if(b == NULL) return 0;
	return (double)(Buffer::ReadX<int32_t>(b));
}
gmexport double buffer_read_uint32(double id) {
	Buffer *b = Buffer::Find(gm_double_to_uint(id));
	if(b == NULL) return 0;
	return (double)(Buffer::ReadX<uint32_t>(b));
}
gmexport double buffer_read_int64(double id) {
	Buffer *b = Buffer::Find(gm_double_to_uint(id));
	if(b == NULL) return 0;
	return (double)(Buffer::ReadX<int64_t>(b));
}
gmexport double buffer_read_uint64(double id) {
	Buffer *b = Buffer::Find(gm_double_to_uint(id));
	if(b == NULL) return 0;
	return (double)(Buffer::ReadX<uint64_t>(b));
}
gmexport double buffer_read_float32(double id) {
	Buffer *b = Buffer::Find(gm_double_to_uint(id));
	if(b == NULL) return 0;
	return (double)(Buffer::ReadX<float>(b));
}
gmexport double buffer_read_float64(double id) {
	Buffer *b = Buffer::Find(gm_double_to_uint(id));
	if(b == NULL) return 0;
	return Buffer::ReadX<double>(b);
}

gmexport double buffer_write_int8(double id, double value) {
	Buffer *b = Buffer::Find(gm_double_to_uint(id));
	if(b == NULL) return 0;
	Buffer::WriteX<int8_t>(b, (int8_t)(gm_double_to_int(value)));
	return 1;
}
gmexport double buffer_write_uint8(double id, double value) {
	Buffer *b = Buffer::Find(gm_double_to_uint(id));
	if(b == NULL) return 0;
	Buffer::WriteX<uint8_t>(b, (uint8_t)(gm_double_to_uint(value)));
	return 1;
}
gmexport double buffer_write_int16(double id, double value) {
	Buffer *b = Buffer::Find(gm_double_to_uint(id));
	if(b == NULL) return 0;
	Buffer::WriteX<int16_t>(b, (int16_t)(gm_double_to_int(value)));
	return 1;
}
gmexport double buffer_write_uint16(double id, double value) {
	Buffer *b = Buffer::Find(gm_double_to_uint(id));
	if(b == NULL) return 0;
	Buffer::WriteX<uint16_t>(b, (uint16_t)(gm_double_to_uint(value)));
	return 1;
}
gmexport double buffer_write_int32(double id, double value) {
	Buffer *b = Buffer::Find(gm_double_to_uint(id));
	if(b == NULL) return 0;
	Buffer::WriteX<int32_t>(b, (int32_t)(gm_double_to_int(value)));
	return 1;
}
gmexport double buffer_write_uint32(double id, double value) {
	Buffer *b = Buffer::Find(gm_double_to_uint(id));
	if(b == NULL) return 0;
	Buffer::WriteX<uint32_t>(b, (uint32_t)(gm_double_to_uint(value)));
	return 1;
}
gmexport double buffer_write_int64(double id, double value) {
	Buffer *b = Buffer::Find(gm_double_to_uint(id));
	if(b == NULL) return 0;
	Buffer::WriteX<int64_t>(b, (int64_t)(gm_double_to_longlong(value)));
	return 1;
}
gmexport double buffer_write_uint64(double id, double value) {
	Buffer *b = Buffer::Find(gm_double_to_uint(id));
	if(b == NULL) return 0;
	Buffer::WriteX<uint64_t>(b, (uint64_t)(gm_double_to_ulonglong(value)));
	return 1;
}
gmexport double buffer_write_float32(double id, double value) {
	Buffer *b = Buffer::Find(gm_double_to_uint(id));
	if(b == NULL) return 0;
	Buffer::WriteX<float>(b, gm_double_to_float(value));
	return 1;
}
gmexport double buffer_write_float64(double id, double value) {
	Buffer *b = Buffer::Find(gm_double_to_uint(id));
	if(b == NULL) return 0;
	Buffer::WriteX<double>(b, value);
	return 1;
}

gmexport const char* buffer_read_string(double id) {
	Buffer *b = Buffer::Find(gm_double_to_uint(id));
	if(b == NULL) return "";
	return Buffer::ReadString(b);
}
gmexport double buffer_write_string(double id, const char* string) {
	Buffer *b = Buffer::Find(gm_double_to_uint(id));
	if(b == NULL) return 0;
	Buffer::WriteString(b, string);
	return 1;
}

gmexport const char* buffer_read_data(double id, double bytes) {
	Buffer *b = Buffer::Find(gm_double_to_uint(id));
	if(b == NULL) return "";
	unsigned int l = gm_double_to_uint(bytes);
	Buffer::SetLength(&returnstring, l);
	Buffer::ReadData(b, Buffer::GetData(&returnstring), l);
	return Buffer::ToString(&returnstring);
}
gmexport double buffer_write_data(double id, const char* string) {
	Buffer *b = Buffer::Find(gm_double_to_uint(id));
	if(b == NULL) return 0;
	Buffer::WriteData(b, string, strlen(string));
	return 1;
}

gmexport const char* buffer_read_hex(double id, double bytes) {
	Buffer *b = Buffer::Find(gm_double_to_uint(id));
	if(b == NULL) return "";
	unsigned int l = gm_double_to_uint(bytes);
	Buffer::SetLength(&returnstring, l * 2);
	Buffer::ReadHex(b, Buffer::GetData(&returnstring), l);
	return Buffer::ToString(&returnstring);
}
gmexport double buffer_write_hex(double id, const char* string) {
	Buffer *b = Buffer::Find(gm_double_to_uint(id));
	if(b == NULL) return 0;
	Buffer::WriteHex(b, string, strlen(string) / 2);
	return 1;
}

gmexport double buffer_write_buffer(double id, double source_id) {
	Buffer *b = Buffer::Find(gm_double_to_uint(id));
	if(b == NULL) return 0;
	Buffer *source = Buffer::Find(gm_double_to_uint(source_id));
	if(source == NULL) return 0;
	Buffer::WriteBuffer(b, source);
	return 1;
}
gmexport double buffer_write_buffer_part(double id, double source_id, double offset, double bytes) {
	Buffer *b = Buffer::Find(gm_double_to_uint(id));
	if(b == NULL) return 0;
	Buffer *source = Buffer::Find(gm_double_to_uint(source_id));
	if(source == NULL) return 0;
	Buffer::WriteBufferPart(b, source, gm_double_to_uint(offset), gm_double_to_uint(bytes));
	return 1;
}

